"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const quota_1 = require("./quota");
const utils_1 = require("./utils");
const Husband = {
    name: 'husband',
    share: function (heirs) {
        if (utils_1.hasChild(heirs)) {
            return quota_1.quarter;
        }
        else {
            return quota_1.half;
        }
    }
};
const Wife = {
    name: 'wife',
    share: function (heirs) {
        if (utils_1.hasChild(heirs)) {
            return quota_1.eighth;
        }
        else {
            return quota_1.quarter;
        }
    }
};
const Daughter = {
    name: 'daughter',
    share: function (heirs) {
        if (utils_1.exists(heirs, 'son')) {
            return quota_1.nothing;
        } // joint ta'seeb
        if (utils_1.count(heirs, this.name) === 1) {
            return quota_1.half;
        }
        else {
            return quota_1.twoThird;
        }
    }
};
const PaternalGrandDaughter = {
    name: 'paternal_grand_daughter',
    share: function (heirs) {
        if (utils_1.exists(heirs, 'son')) {
            return quota_1.nothing;
        }
        if (utils_1.exists(heirs, 'paternal_grand_son')) {
            return quota_1.nothing;
        } // joint ta'seeb
        if (utils_1.count(heirs, 'daughter') > 1) {
            return quota_1.nothing;
        }
        else if (utils_1.count(heirs, 'daughter') === 1) {
            return quota_1.sixth;
        }
        else {
            if (utils_1.count(heirs, this.name) === 1) {
                return quota_1.half;
            }
            else {
                return quota_1.twoThird;
            }
        }
    }
};
const Father = {
    name: 'father',
    share: function (heirs) {
        if (utils_1.hasChild(heirs)) {
            return quota_1.sixth;
        }
        return quota_1.nothing; // ta'seeb
    }
};
const Mother = {
    name: 'mother',
    share: function (heirs) {
        if (utils_1.hasChild(heirs)) {
            return quota_1.sixth;
        }
        if (utils_1.hasGroupOfSiblings(heirs)) {
            return quota_1.sixth;
        }
        return quota_1.third;
    }
};
const PaternalGrandFather = {
    name: 'paternal_grand_father',
    share: function (heirs) {
        if (utils_1.exists(heirs, 'father')) {
            return quota_1.nothing;
        }
        if (utils_1.hasChild(heirs)) {
            return quota_1.sixth;
        }
        return quota_1.nothing; // ta'seeb
    }
};
const PaternalGrandMother = {
    name: 'paternal_grand_mother',
    share: function (heirs) {
        if (utils_1.exists(heirs, 'father')) {
            return quota_1.nothing;
        }
        if (utils_1.exists(heirs, 'mother')) {
            return quota_1.nothing;
        }
        return quota_1.sixth; // NOTE: sixth is shared with maternal_grand_mother
    }
};
const MaternalGrandMother = {
    name: 'maternal_grand_mother',
    share: function (heirs) {
        if (utils_1.exists(heirs, 'mother')) {
            return quota_1.nothing;
        }
        return quota_1.sixth; // NOTE: sixth is shared with paternal_grand_mother
    }
};
const FullSister = {
    name: 'full_sister',
    share: function (heirs) {
        if (utils_1.hasChild(heirs)) {
            return quota_1.nothing;
        }
        if (utils_1.hasPaternalMaleAncestor(heirs)) {
            return quota_1.nothing;
        }
        if (utils_1.exists(heirs, 'full_brother')) {
            return quota_1.nothing;
        } // joint ta'seeb
        if (utils_1.count(heirs, this.name) === 1) {
            return quota_1.half;
        }
        else {
            return quota_1.twoThird;
        }
    }
};
const PaternalSister = {
    name: 'paternal_sister',
    share: function (heirs) {
        if (utils_1.hasChild(heirs)) {
            return quota_1.nothing;
        }
        if (utils_1.hasPaternalMaleAncestor(heirs)) {
            return quota_1.nothing;
        }
        if (utils_1.exists(heirs, 'full_brother')) {
            return quota_1.nothing;
        }
        if (utils_1.exists(heirs, 'paternal_brother')) {
            return quota_1.nothing;
        } // joint ta'seeb
        if (utils_1.count(heirs, 'full_sister') > 1) {
            return quota_1.nothing;
        }
        if (utils_1.count(heirs, 'full_sister') === 1) {
            if (utils_1.count(heirs, this.name) === 1) {
                return quota_1.sixth;
            }
            else {
                return quota_1.nothing;
            }
        }
        else {
            if (utils_1.count(heirs, this.name) === 1) {
                return quota_1.half;
            }
            else {
                return quota_1.twoThird;
            }
        }
    }
};
const MaternalSibling = {
    name: 'maternal_sibling',
    share: function (heirs) {
        if (utils_1.hasChild(heirs)) {
            return quota_1.nothing;
        }
        if (utils_1.hasPaternalMaleAncestor(heirs)) {
            return quota_1.nothing;
        }
        if (utils_1.count(heirs, this.name) === 1) {
            return quota_1.sixth;
        }
        else {
            return quota_1.third;
        }
    }
};
const fhs = [
    Daughter,
    Father,
    FullSister,
    Husband,
    MaternalGrandMother,
    MaternalSibling,
    Mother,
    PaternalGrandDaughter,
    PaternalGrandFather,
    PaternalGrandMother,
    PaternalSister,
    Wife
];
exports.default = fhs;
//# sourceMappingURL=fardHeirs.js.map