"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateSpecialCases = void 0;
const fraction_js_1 = __importDefault(require("fraction.js"));
const lodash_1 = require("lodash");
const result_1 = require("./result");
const quota_1 = require("./quota");
const utils_1 = require("./utils");
function calculateSpecialCases(fardResult, asabaResult) {
    const results = [...fardResult, ...asabaResult];
    return lodash_1.flow([umariyyahCase, mushtarakaCase, awlCase, raddCase])(results);
}
exports.calculateSpecialCases = calculateSpecialCases;
function awlCase(result) {
    const whole = new fraction_js_1.default(1);
    const remaining = whole.sub(result_1.sumResults(result));
    const sum = result_1.sumResults(result);
    if (remaining.compare(0) < 0) {
        return result.map(r => (Object.assign(Object.assign({}, r), { share: r.share.div(sum) })));
    }
    return result;
}
function raddCase(result) {
    const whole = new fraction_js_1.default(1);
    const remaining = whole.sub(result_1.sumResults(result));
    if (remaining.compare(0) > 0) {
        const ratios = toRatio(result.map(r => {
            if ((r.name === 'wife' || r.name === 'husband') && result.length > 1) {
                return new fraction_js_1.default(0);
            }
            return r.share;
        }));
        return lodash_1.zip(result, ratios).map(([r, ratio]) => {
            if (!r || !ratio) {
                throw Error('result and ratios should be equal in lenght');
            }
            return Object.assign(Object.assign({}, r), { share: r.share.add(remaining.mul(ratio)) });
        });
    }
    return result;
}
function mushtarakaCase(result) {
    const fullBrother = result_1.findFromResult(result, 'full_brother');
    const maternalSibling = result_1.findFromResult(result, 'maternal_sibling');
    if (fullBrother && maternalSibling) {
        if (fullBrother.share.compare(quota_1.nothing) === 0) {
            return result_1.updateResults(result, utils_1.distribute([fullBrother, maternalSibling], maternalSibling.share));
        }
    }
    return result;
}
function umariyyahCase(result) {
    const father = result_1.findFromResult(result, 'father');
    const mother = result_1.findFromResult(result, 'mother');
    const wife = result_1.findFromResult(result, 'wife');
    const husband = result_1.findFromResult(result, 'husband');
    const isUmariyyah = result.every(r => {
        const umariyyahParticipants = ['father', 'mother', 'husband', 'wife'];
        return umariyyahParticipants.includes(r.name);
    });
    if (!isUmariyyah)
        return result;
    const type = 'special_case';
    if (father && mother && wife) {
        return [
            Object.assign(Object.assign({}, wife), { share: quota_1.quarter }),
            Object.assign(Object.assign({}, father), { type, share: quota_1.half }),
            Object.assign(Object.assign({}, mother), { type, share: quota_1.quarter })
        ];
    }
    if (father && mother && husband) {
        return [
            Object.assign(Object.assign({}, husband), { share: quota_1.half }),
            Object.assign(Object.assign({}, father), { type, share: quota_1.third }),
            Object.assign(Object.assign({}, mother), { type, share: quota_1.sixth })
        ];
    }
    return result;
}
const toRatio = (fractions) => {
    const oldBase = fractions.reduce((accumulator, current) => accumulator.gcd(current), new fraction_js_1.default(1)).d;
    const ratios = fractions.map(f => (oldBase / f.d) * f.n);
    const newBase = lodash_1.sum(ratios);
    return ratios.map(r => new fraction_js_1.default(r, newBase));
};
//# sourceMappingURL=specialCases.js.map