"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateTasib = void 0;
const asabaHeirs_1 = __importDefault(require("./asabaHeirs"));
const quota_1 = require("./quota");
const result_1 = require("./result");
const utils_1 = require("./utils");
const fraction_js_1 = __importDefault(require("fraction.js"));
function calculateTasib(heirs, fardResult) {
    // filter out asaba that exist and sort them by their tasibRank
    const asabas = asabaHeirs_1.default
        .filter(ah => utils_1.exists(heirs, ah.name))
        .filter(ah => {
        // If an heir is given the prescribed share, he/she drops from Ta’seeb
        // father is an exception to this rule 
        return (!fardResult.find(fh => fh.name === ah.name) ||
            ah.name === 'father');
    });
    const qualifiedAsabas = asabas
        .filter(ah => asabas[0].tasibRank === ah.tasibRank);
    const results = qualifiedAsabas
        .map(ah => {
        const result = {
            name: ah.name,
            count: utils_1.count(heirs, ah.name),
            type: 'tasib',
            share: quota_1.unknown
        };
        return result;
    });
    const whole = new fraction_js_1.default(1);
    let remaining = whole.sub(result_1.sumResults(fardResult));
    if (remaining.compare(0) < 0) {
        remaining = quota_1.nothing;
    }
    switch (results.length) {
        case 0: return results;
        case 1: return utils_1.distribute(results, remaining);
        case 2: return jointTasib(results, remaining);
        default: throw Error('qualified asaba types cannot be greater than two');
    }
}
exports.calculateTasib = calculateTasib;
// takes a pair of asaba result where the first one is a male and the second
// is a female and distributes it in 2:1 ratio
const jointTasib = (results, remaining) => {
    return utils_1.distribute([Object.assign(Object.assign({}, results[0]), { proportion: 2 }), Object.assign(Object.assign({}, results[1]), { proportion: 1 })], remaining);
};
//# sourceMappingURL=tasib.js.map