"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isZero = exports.distribute = exports.hasPaternalMaleAncestor = exports.hasGroupOfSiblings = exports.hasChild = exports.exists = exports.count = void 0;
const fraction_js_1 = __importDefault(require("fraction.js"));
const lodash_1 = require("lodash");
const count = (heirs, heir) => heirs[heir];
exports.count = count;
const exists = (heirs, heir) => heirs[heir] > 0;
exports.exists = exists;
const hasChild = (heirs) => {
    const children = ['son', 'daughter', 'paternal_grand_son', 'paternal_grand_daughter'];
    return children.some(child => exports.exists(heirs, child));
};
exports.hasChild = hasChild;
const hasGroupOfSiblings = (heirs) => {
    const siblings = [
        'full_brother', 'full_sister', 'paternal_sister', 'paternal_brother',
        'maternal_sibling'
    ];
    let count = 0;
    siblings.forEach(sibling => count += heirs[sibling]);
    return count > 1;
};
exports.hasGroupOfSiblings = hasGroupOfSiblings;
const hasPaternalMaleAncestor = (heirs) => {
    const ancestors = ['father', 'paternal_grand_father'];
    return ancestors.some(ancestor => exports.exists(heirs, ancestor));
};
exports.hasPaternalMaleAncestor = hasPaternalMaleAncestor;
const distribute = (heirs, amount) => {
    const total = lodash_1.sumBy(heirs, h => (h.proportion || 1) * h.count);
    return heirs.map(h => {
        const result = {
            name: h.name,
            count: h.count,
            type: h.type,
            share: amount.mul((h.proportion || 1) * h.count, total)
        };
        return result;
    });
};
exports.distribute = distribute;
const isZero = (v) => v.equals(new fraction_js_1.default(0));
exports.isZero = isZero;
//# sourceMappingURL=utils.js.map